@@ -8,7 +8,7 @@ import logging
 
 from types import SimpleNamespace
 
-# logger for background processes (pinger etc.)
+# logger for background processes (pinger and data stream)
 logger = logging.getLogger('psrt')
 
 DEFAULT_TIMEOUT = 5
@@ -21,7 +21,7 @@ OP_NOP = b'\x00'
 OP_BYE = b'\xFF'
 
 OP_PUBLISH = b'\x01\x7F'  # priority hard-coded until supported
-OP_PUBLISH_NO_ACK = b'\x12\x7F'  # priority hard-coded until supported
+OP_PUBLISH_NO_ACK = b'\x21\x7F'  # priority hard-coded until supported
 OP_SUBSCRIBE = b'\x02'
 OP_UNSUBSCRIBE = b'\x03'
 
@@ -32,10 +32,19 @@ CONTROL_HEADER = b'\xEE\xAA'
 DATA_HEADER = b'\xEE\xAB'
 
 
-def pub_udp(target, topic, message, need_ack=True, **kwargs):
+def pub_udp(target,
+            topic,
+            message,
+            need_ack=True,
+            check_ack_src=True,
+            **kwargs):
     if isinstance(target, str):
         host, port = target.rsplit(':', maxsplit=1)
+        if check_ack_src:
+            host = socket.gethostbyname(host)
         target = (host, int(port))
+    elif check_ack_src:
+        target = (socket.gethostbyname(target[0]), target[1])
     user = kwargs.get('user', '')
     password = kwargs.get('password', '')
     timeout = kwargs.get('timeout', DEFAULT_TIMEOUT)
@@ -54,11 +63,14 @@ def pub_udp(target, topic, message, need_ack=True, **kwargs):
         message, target)
     if need_ack:
         client_socket.settimeout(timeout)
-        (data, server) = client_socket.recvfrom(1)
-        print(server, target)
-        if server != target:
-            raise RuntimeError('Invalid responder')
-        code = data[0]
+        (data, server) = client_socket.recvfrom(5)
+        if check_ack_src and server != target:
+            raise RuntimeError(f'Invalid ack source: {server}')
+        if data[0:2] != CONTROL_HEADER:
+            raise RuntimeError(f'Invalid control header in ack')
+        if int.from_bytes(data[2:4], 'little') != PROTO_VERSION:
+            raise RuntimeError(f'Invalid server protocol in ack')
+        code = data[4]
         if code == RESPONSE_ACCESS_DENIED:
             raise AccessError
         elif code != RESPONSE_OK:
